@@ -1,20 +1,27 @@
 import sys
 import os
+import ast
+from collections import namedtuple
 import code
 import itertools
 import tokenize
 import unicodedata
 from unittest.mock import patch
+from pathlib import Path
 
 from typing import Optional, List, Dict, Any, TYPE_CHECKING
+from Orange.widgets import widget
 
 import pygments.style
 from pygments.token import Comment, Keyword, Number, String, Punctuation, Operator, Error, Name
 from qtconsole.pygments_highlighter import PygmentsHighlighter
 
+from orangewidget.utils.combobox import ComboBoxSearch
+from Orange.widgets.utils.itemmodels import DomainModel
 from AnyQt.QtWidgets import (
     QPlainTextEdit, QListView, QSizePolicy, QMenu, QSplitter, QLineEdit,
-    QAction, QToolButton, QFileDialog, QStyledItemDelegate,
+    QAction, QToolButton, QFileDialog, QStyledItemDelegate, 
+    QTableWidget, QHeaderView, QComboBox,
     QStyleOptionViewItem, QPlainTextDocumentLayout,
     QLabel, QWidget, QHBoxLayout, QApplication)
 from AnyQt.QtGui import (
@@ -42,8 +49,9 @@ if TYPE_CHECKING:
 
 __all__ = ["OWPythonScript"]
 
+AddOpts = namedtuple("AddOpts", ["name", "value", "is_default"])
 
-DEFAULT_SCRIPT = """import numpy as np
+NUMPY_SCRIPT = """import numpy as np
 from Orange.data import Table, Domain, ContinuousVariable, DiscreteVariable
 
 domain = Domain([ContinuousVariable("age"),
@@ -55,6 +63,23 @@ arr = np.array([
 out_data = Table.from_numpy(domain, arr)
 """
 
+PANDAS_SCRIPT = """import numpy as np
+import pandas as pd
+from Orange.data.pandas_compat import table_from_frame,table_to_frame
+from Orange.data import Domain, Table
+
+df = table_to_frame(in_data) # 你的数据就叫 df 了
+
+############## 以下写入你自己的代码 #####################################
+
+
+
+
+############# 以上写入你自己的代码 #####################################
+
+out_data = table_from_frame(df) # 将你的数据输出
+"""
+
 
 def text_format(foreground=Qt.black, weight=QFont.Normal):
     fmt = QTextCharFormat()
@@ -215,8 +240,8 @@ class ReturnStatement(FakeSignatureMixin, QWidget):
         layout.setSpacing(0)
 
         # `return `
-        ret_lbl = QLabel('<b style="color: ' +
-                         highlighting_scheme[Keyword].split(' ')[-1] +
+        ret_lbl = QLabel('<b style="color: ' + \
+                         highlighting_scheme[Keyword].split(' ')[-1] + \
                          ';">return </b>', self)
         ret_lbl.setFont(self.font())
         ret_lbl.setContentsMargins(0, 0, 0, 0)
@@ -287,8 +312,8 @@ class VimIndicator(QWidget):
         p.drawRoundedRect(rect, 5, 5)
         p.restore()
 
-        textstart = (width - fm.width(self.indicator_text)) / 2
-        p.drawText(textstart, height / 2 + 5, self.indicator_text)
+        textstart = (width - fm.width(self.indicator_text)) // 2
+        p.drawText(textstart, height // 2 + 5, self.indicator_text)
 
     def minimumSizeHint(self):
         fm = QFontMetrics(self.font())
@@ -314,6 +339,9 @@ class PythonConsole(QPlainTextEdit, code.InteractiveConsole):
     def updateLocals(self, locals):
         self.locals.update(locals)
 
+    def clearInParams(self):
+        self.locals['in_params'] = {}
+
     def interact(self, banner=None, _=None):
         try:
             sys.ps1
@@ -549,12 +577,19 @@ class OWPythonScript(OWWidget):
 
     signal_names = ("data", "learner", "classifier", "object")
 
-    settings_version = 2
-    scriptLibrary: 'List[_ScriptData]' = Setting([{
-        "name": "Table from numpy",
-        "script": DEFAULT_SCRIPT,
+    settings_version = 3
+    scriptLibrary: 'List[_ScriptData]' = Setting([
+    {
+        "name": "Table from pandas",
+        "script": PANDAS_SCRIPT,
         "filename": None
-    }])
+    },
+     {
+        "name": "Table from numpy",
+        "script": NUMPY_SCRIPT,
+        "filename": None
+    }
+    ])
     currentScriptIndex = Setting(0)
     scriptText: Optional[str] = Setting(None, schema_only=True)
     splitterState: Optional[bytes] = Setting(None)
@@ -567,6 +602,9 @@ class OWPythonScript(OWWidget):
     def __init__(self):
         super().__init__()
 
+        # name, discription, options
+        self.additional_controls = []
+
         for name in self.signal_names:
             setattr(self, name, [])
 
@@ -666,7 +704,6 @@ class OWPythonScript(OWWidget):
             callback=enable_vim_mode
         )
         self.vim_box.layout().addWidget(self.vim_indicator)
-
         @editor.vimModeIndicationChanged.connect
         def _(color, text):
             self.vim_indicator.indicator_color = color
@@ -741,9 +778,16 @@ class OWPythonScript(OWWidget):
 
         self.controlBox.layout().addWidget(w)
 
-        self.execute_button = gui.button(
-            self.buttonsArea, self, '运行', callback=self.commit)
+        box = gui.vBox(self.controlArea, '脚本参数', stretch=100)
+        self.add_ui_list = QTableWidget(
+            box, showGrid=False, selectionMode=QTableWidget.NoSelection)
+        box.layout().addWidget(self.add_ui_list)
+        self.add_ui_list.setColumnCount(2)
+        self.add_ui_list.setRowCount(0)
+        self.add_ui_list.verticalHeader().hide()
+        self.add_ui_list.horizontalHeader().hide()
 
+        self.execute_button = gui.button(self.buttonsArea, self, '运行', callback=self.commit)
         self.run_action = QAction("Run script", self, triggered=self.commit,
                                   shortcut=QKeySequence(Qt.ControlModifier | Qt.Key_R))
         self.addAction(self.run_action)
@@ -872,9 +916,13 @@ class OWPythonScript(OWWidget):
         self.setSelectedScript(len(self.libraryList) - 1)
 
     def onAddScriptFromFile(self, *_):
+        dir_path = Path(__file__).resolve()
+        parent_path = dir_path.parent.parent.parent
+        scriptStore = f'{str(parent_path)}/scriptStore'
+
         filename, _ = QFileDialog.getOpenFileName(
             self, 'Open Python Script',
-            os.path.expanduser("~/"),
+            os.path.expanduser(scriptStore),
             'Python files (*.py)\nAll files(*.*)'
         )
         if filename:
@@ -883,6 +931,106 @@ class OWPythonScript(OWWidget):
                 contents = f.read()
             self.libraryList.append(Script(name, contents, 0, filename))
             self.setSelectedScript(len(self.libraryList) - 1)
+            # self.make_gui_controls_from_doc_string(contents)
+
+    def make_gui_controls_from_doc_string(self, contents):
+        self.clear_additional_controls()
+        try:
+            doc_string = contents.split("'''")[1]
+        except IndexError:
+            print('没有参数吧？')
+            return
+        self._parse_doc_string(doc_string)
+
+        for control in self.additional_controls:
+            # opt_name_list = [option.name for option in control['options']]
+            if len(control['options']) == 1: # 填空题
+                self._add_row(control['name'], control['description'], control['options'], to_choose=False)
+            else:
+                self._add_row(control['name'], control['description'], control['options'])
+
+
+    def _parse_doc_string(self, doc_string):
+        """
+        将doc_string解析为一个字典, 分别为name, description 和 options
+        其中, options为一个namedtuple("AddOpts", ["name", "value", "is_default"])
+        """
+
+
+        # 添加新的
+        for line in doc_string.split('\n'):
+            if not line.startswith('-'):
+                continue
+            if line == '':
+                continue
+            try:
+                name, description, options = line.split('|')
+            except ValueError:
+                return
+
+            name = name.split('-')[1].strip()
+            discription = description.strip()
+            options = ast.literal_eval(options)
+
+            # 设置默认选项     
+            if (len(options) == 1): # 填空题
+                setattr(self, name, options[0])
+            else: # 选择题
+                options = [AddOpts._make(opt) for opt in options]
+                for i, opt in enumerate(options):
+                    setattr(self, name, 0)
+                    if opt.is_default:
+                        setattr(self, name, i)
+                        break
+            
+            self.additional_controls.append({'name': name, 'description': description, 'options': options})
+
+    def _add_row(self, name, description, options, to_choose=True):
+        model = self.add_ui_list.model()
+        row = model.rowCount()
+        model.insertRow(row)
+
+        # self.add_ui_list
+        label = gui.label(None, self, description, tooltip=description)
+        self.add_ui_list.setCellWidget(row, 0, label)
+
+        if to_choose: # 选择题
+            opt_name = [opt.name for opt in options]
+            opt_values = [opt.value for opt in options]
+            opt_is_default = [opt.is_default for opt in options]
+
+            oper_combo = QComboBox()
+            # oper_combo.row = row
+            oper_combo.addItems(opt_name)
+            oper_combo.setCurrentIndex(opt_is_default.index(True))
+
+            self.add_ui_list.setCellWidget(row, 1, oper_combo)
+            oper_combo.currentIndexChanged.connect(
+                lambda index: setattr(self, name, index))
+        else: # 填空题
+            line_edit = QLineEdit()
+            line_edit.setPlaceholderText(description)
+            line_edit.setText(f'{options[0]}')
+            line_edit.textChanged.connect(lambda text: setattr(self, name, text))
+            self.add_ui_list.setCellWidget(row, 1, line_edit)
+
+        # self.add_ui_list.resizeColumnsToContents()
+
+
+    def clear_additional_controls(self):
+        self.additional_controls = []
+        for row in range(self.add_ui_list.rowCount()):
+            widg = self.add_ui_list.cellWidget(row, 1)
+            if widg and type(widget) == QComboBox:
+                widg.currentIndexChanged.disconnect()
+            elif widg and type(widget) == QLineEdit:
+                widg.textChanged.disconnect()
+
+        add_names = [control['name'] for control in self.additional_controls]
+        for n in add_names:
+            delattr(self, n)
+        self.add_ui_list.clear()
+        self.add_ui_list.setRowCount(0)
 
     def onRemoveScript(self, *_):
         index = self.selectedScriptIndex()
@@ -905,6 +1053,11 @@ class OWPythonScript(OWWidget):
 
             self.text.setDocument(self.documentForScript(current))
             self.currentScriptIndex = current
+            self.make_gui_controls_from_doc_string(self.libraryList[current].script)
+            self.clear_in_params()
+            
+    def clear_in_params(self):
+        self.console.clearInParams()
 
     def documentForScript(self, script=0):
         if not isinstance(script, Script):
@@ -916,8 +1069,7 @@ class OWPythonScript(OWWidget):
             doc.setDefaultFont(QFont(self.defaultFont))
             doc.highlighter = PygmentsHighlighter(doc)
             doc.highlighter.set_style(self.pygments_style_class)
-            doc.setDefaultFont(
-                QFont(self.defaultFont, pointSize=self.defaultFontSize))
+            doc.setDefaultFont(QFont(self.defaultFont, pointSize=self.defaultFontSize))
             doc.modificationChanged[bool].connect(self.onModificationChanged)
             doc.setModified(False)
             self._cachedDocuments[script] = doc
@@ -964,18 +1116,25 @@ class OWPythonScript(OWWidget):
             else:
                 fn = filename
 
-            f = open(fn, 'w')
+            f = open(fn, 'w', encoding='utf-8')
             f.write(self.text.toPlainText())
             f.close()
 
     def initial_locals_state(self):
-        d = {}
+        d = {'in_params': {}}
         for name in self.signal_names:
             value = getattr(self, name)
             all_values = list(value)
             one_value = all_values[0] if len(all_values) == 1 else None
             d["in_" + name + "s"] = all_values
             d["in_" + name] = one_value
+        for control in self.additional_controls:
+            name = control['name']
+            v = getattr(self, name)
+            if len(control['options']) == 1: # 填空题
+                 d["in_params"][name] = v
+            else:
+                d["in_params"][name] = control['options'][v].value
         return d
 
     def commit(self):
