@@ -30,7 +30,6 @@ from Orange.widgets.data.owcsvimport import (
 )
 from Orange.widgets.utils.pathutils import PathItem, samepath
 from Orange.widgets.utils.settings import QSettings_writeArray
-from Orange.widgets.utils.state_summary import format_summary_details
 
 W = TypeVar("W", bound=OWBaseWidget)
 
@@ -147,25 +146,6 @@ class TestOWCSVFileImport(WidgetTest):
         )
         self._check_data_regions(self.get_output("数据(Data)", w))
 
-    def test_summary(self):
-        """Check if status bar is updated when data is received"""
-        dirname = os.path.dirname(__file__)
-        path = os.path.join(dirname, "data-regions.tab")
-        widget = self.create_widget(
-            owcsvimport.OWCSVFileImport,
-            stored_settings={
-                "_session_items": [
-                    (path, self.data_regions_options.as_dict())
-                ]
-            }
-        )
-        output_sum = widget.info.set_output_summary = mock.Mock()
-        widget.commit()
-        self.wait_until_finished(widget)
-        output = self.get_output("数据(Data)", widget)
-        output_sum.assert_called_with(len(output),
-                                      format_summary_details(output))
-
     data_csv_types_options = owcsvimport.Options(
         encoding="ascii", dialect=csv.excel_tab(),
         columntypes=[
@@ -259,9 +239,9 @@ class TestOWCSVFileImport(WidgetTest):
             dlg = browse_dialog()
             dlg.setOption(QFileDialog.DontUseNativeDialog)
             dlg.selectFile(path)
-            dlg.exec_ = dlg.exec = lambda: QFileDialog.Accepted
+            dlg.exec = lambda: QFileDialog.Accepted
             r.return_value = dlg
-            with mock.patch.object(owcsvimport.CSVImportDialog, "exec_",
+            with mock.patch.object(owcsvimport.CSVImportDialog, "exec",
                                    lambda _: QFileDialog.Accepted):
                 yield
 
