@@ -1,12 +1,16 @@
+# pylint: disable=unsubscriptable-object
 import unittest
 import ast
 import sys
 import math
 import pickle
 import copy
+from unittest.mock import patch, Mock
 
 import numpy as np
 
+from orangewidget.settings import Context
+
 from Orange.data import (Table, Domain, StringVariable,
                          ContinuousVariable, DiscreteVariable, TimeVariable)
 from Orange.widgets.tests.base import WidgetTest
@@ -32,14 +36,14 @@ class FeatureConstructorTest(unittest.TestCase):
         values = ('iris one', 'iris two', 'iris three')
         desc = PyListModel(
             [DiscreteDescriptor(name=name, expression=expression,
-                                values=values, ordered=False)]
+                                values=values, ordered=True)]
         )
         data = data.transform(Domain(list(data.domain.attributes) +
                                      construct_variables(desc, data),
                                      data.domain.class_vars,
                                      data.domain.metas))
         self.assertTrue(isinstance(data.domain[name], DiscreteVariable))
-        self.assertEqual(data.domain[name].values, list(values))
+        self.assertEqual(data.domain[name].values, values)
         for i in range(3):
             self.assertEqual(data[i * 50, name], values[i])
 
@@ -93,7 +97,7 @@ class FeatureConstructorTest(unittest.TestCase):
                                      data.domain.metas))
         self.assertTrue(isinstance(data.domain[name], TimeVariable))
         for row in data:
-            self.assertEqual("2019-07-{:02}".format(int(row["MEDV"] / 3)),
+            self.assertEqual(f"2019-07-{int(row['MEDV'] / 3):02}",
                              str(row["Date"])[:10])
 
     def test_construct_variables_string(self):
@@ -128,6 +132,22 @@ class FeatureConstructorTest(unittest.TestCase):
         np.testing.assert_array_equal(ndata.X[:, 0],
                                       data.X[:, :2].sum(axis=1))
 
+    @staticmethod
+    def test_unicode_normalization():
+        micro = "\u00b5"
+        domain = Domain([ContinuousVariable(micro)])
+        name = 'Micro Variable'
+        expression = micro
+        desc = PyListModel(
+            [ContinuousDescriptor(name=name, expression=expression,
+                                  number_of_decimals=2)]
+        )
+        data = Table.from_numpy(domain, np.arange(5).reshape(5, 1))
+        data = data.transform(Domain(data.domain.attributes,
+                                     [],
+                                     construct_variables(desc, data)))
+        np.testing.assert_equal(data.X, data.metas)
+
 
 class TestTools(unittest.TestCase):
     def test_free_vars(self):
@@ -274,10 +294,46 @@ class FeatureFuncTest(unittest.TestCase):
         self.assertEqual(r, [x[0] for x in zoo.metas[:, 0]])
         self.assertEqual(f(zoo[0]), str(zoo[0, "name"])[0])
 
+    def test_missing_variable(self):
+        zoo = Table("zoo")
+        assert zoo.domain.class_var.name == "type"
+        f = FeatureFunc("type[0]",
+                        [("type", zoo.domain["type"])])
+        no_class = Domain(zoo.domain.attributes, None, zoo.domain.metas)
+        data2 = zoo.transform(no_class)
+        r = f(data2)
+        self.assertTrue(np.all(np.isnan(r)))
+        self.assertTrue(np.isnan(f(data2[0])))
+
+    def test_time_str(self):
+        data = Table.from_numpy(Domain([TimeVariable("T", have_date=True)]), [[0], [0]])
+        f = FeatureFunc("str(T)", [("T", data.domain[0])])
+        c = f(data)
+        self.assertEqual(c, ["1970-01-01", "1970-01-01"])
+
+    def test_invalid_expression_variable(self):
+        iris = Table("iris")
+        f = FeatureFunc("1 / petal_length",
+                        [("petal_length", iris.domain["petal length"])])
+        with iris.unlocked():
+            iris[0]["petal length"] = 0
+
+        f.mask_exceptions = False
+        self.assertRaises(Exception, f, iris)
+        self.assertRaises(Exception, f, iris[0])
+        _ = f(iris[1])
+
+        f.mask_exceptions = True
+        r = f(iris)
+        self.assertTrue(np.isnan(r[0]))
+        self.assertFalse(np.isnan(r[1]))
+        self.assertTrue(np.isnan(f(iris[0])))
+        self.assertFalse(np.isnan(f(iris[1])))
+
 
 class OWFeatureConstructorTests(WidgetTest):
     def setUp(self):
-        self.widget = OWFeatureConstructor()
+        self.widget = self.create_widget(OWFeatureConstructor)
 
     def test_create_variable_with_no_data(self):
         self.widget.addFeature(
@@ -297,6 +353,30 @@ class OWFeatureConstructorTests(WidgetTest):
         self.widget.apply()
         self.assertTrue(self.widget.Error.invalid_expressions.is_shown())
 
+    def test_transform_error(self):
+        data = Table("iris")[::5]
+        self.send_signal(self.widget.Inputs.data, data)
+        self.widget.addFeature(ContinuousDescriptor("X", "1/0", 3))
+        self.widget.apply()
+        self.wait_until_finished(self.widget)
+        self.assertTrue(self.widget.Error.transform_error.is_shown())
+        self.widget.removeFeature(0)
+        self.widget.addFeature(ContinuousDescriptor("X", "1", 3))
+        self.widget.apply()
+        self.wait_until_finished(self.widget)
+        self.assertFalse(self.widget.Error.transform_error.is_shown())
+
+    def test_renaming_duplicate_vars(self):
+        data = Table("iris")
+        self.widget.setData(data)
+        self.widget.addFeature(
+            ContinuousDescriptor("iris", "0", 3)
+        )
+        self.widget.apply()
+        output = self.get_output(self.widget.Outputs.data)
+        self.assertEqual(len(set(var.name for var in output.domain.variables)),
+                         len(output.domain.variables))
+
     def test_discrete_no_values(self):
         """
         Should not fail when there are no values set.
@@ -308,14 +388,134 @@ class OWFeatureConstructorTests(WidgetTest):
 
         discreteFeatureEditor.valuesedit.setText("A")
         discreteFeatureEditor.nameedit.setText("D1")
-        discreteFeatureEditor.expressionedit.setText("iris")
+        discreteFeatureEditor.expressionedit.setText("1")
         self.widget.addFeature(
             discreteFeatureEditor.editorData()
         )
         self.assertFalse(self.widget.Error.more_values_needed.is_shown())
         self.widget.apply()
+        self.wait_until_finished(self.widget)
         self.assertTrue(self.widget.Error.more_values_needed.is_shown())
 
+    @patch("Orange.widgets.data.owfeatureconstructor.QMessageBox")
+    def test_fix_values(self, msgbox):
+        w = self.widget
+
+        msgbox.ApplyRole, msgbox.RejectRole = object(), object()
+        msgbox.return_value = Mock()
+        dlgexec = msgbox.return_value.exec = Mock()
+
+        v = [DiscreteVariable(name, values=tuple("abc"))
+             for name in ("ana", "berta", "cilka")]
+        domain = Domain(v, [])
+        self.send_signal(w.Inputs.data, Table.from_numpy(domain, [[0, 1, 2]]))
+
+        w.descriptors = [StringDescriptor(
+            "y", "ana.value + berta.value + cilka.value")]
+
+        # Reject fixing - no changes
+        dlgexec.return_value=msgbox.RejectRole
+        w.fix_expressions()
+        self.assertEqual(w.descriptors[0].expression,
+                         "ana.value + berta.value + cilka.value")
+
+        dlgexec.return_value = Mock(return_value=msgbox.AcceptRole)
+
+        w.fix_expressions()
+        self.assertEqual(w.descriptors[0].expression, "ana + berta + cilka")
+
+        w.descriptors = [StringDescriptor(
+            "y", "ana.value + dani.value + cilka.value")]
+        with patch.object(w, "apply"):  # dani doesn't exist and will fail
+            w.fix_expressions()
+        self.assertEqual(w.descriptors[0].expression,
+                         "ana + dani.value + cilka")
+
+        w.descriptors = [ContinuousDescriptor("y", "sqrt(berta)", 1)]
+        w.fix_expressions()
+        self.assertEqual(w.descriptors[0].expression,
+                         "sqrt({'a': 0, 'b': 1, 'c': 2}[berta])")
+
+    def test_migration_discrete_strings(self):
+        v = [DiscreteVariable("Ana", values=tuple("012")),
+             ContinuousVariable("Cilka")]
+        domain = Domain(v)
+        data = Table.from_numpy(domain, [[1, 3.14]])
+
+        settings_w_discrete = {
+            "context_settings":
+            [Context(
+                attributes=dict(Ana=1, Cilka=2), metas={},
+                values=dict(
+                    descriptors=[
+                        ContinuousDescriptor("y", "Ana + int(Cilka)", 1),
+                        StringDescriptor("u", "Ana.value + 'X'")
+                    ],
+                    currentIndex=0)
+             )]
+        }
+        widget = self.create_widget(OWFeatureConstructor, settings_w_discrete)
+        self.send_signal(widget.Inputs.data, data)
+        self.assertTrue(widget.expressions_with_values)
+        self.assertFalse(widget.fix_button.isHidden())
+        out = self.get_output(widget.Outputs.data)
+        np.testing.assert_almost_equal(out.X, [[1, 3.14, 4]])
+        np.testing.assert_equal(out.metas, [["1X"]])
+
+        settings_no_discrete = {
+            "context_settings":
+            [Context(
+                attributes=dict(Ana=1, Cilka=2), metas={},
+                values=dict(
+                    descriptors=[
+                        ContinuousDescriptor("y", "int(Cilka)", 1),
+                    ],
+                    currentIndex=0)
+             )]
+        }
+        widget = self.create_widget(OWFeatureConstructor, settings_no_discrete)
+        self.send_signal(widget.Inputs.data, data)
+        self.assertFalse(widget.expressions_with_values)
+        self.assertTrue(widget.fix_button.isHidden())
+        out = self.get_output(widget.Outputs.data)
+        np.testing.assert_almost_equal(out.X, [[1, 3.14, 3]])
+
+        widget = self.create_widget(OWFeatureConstructor, settings_w_discrete)
+        self.send_signal(widget.Inputs.data, data)
+        self.assertTrue(widget.expressions_with_values)
+        self.assertFalse(widget.fix_button.isHidden())
+        self.send_signal(widget.Inputs.data, None)
+        self.assertFalse(widget.expressions_with_values)
+        self.assertTrue(widget.fix_button.isHidden())
+
+    def test_report(self):
+        settings = {
+            "context_settings":
+                [Context(
+                    attributes=dict(x=2, y=2, z=2), metas={},
+                    values=dict(
+                        descriptors=[
+                            ContinuousDescriptor("a", "x + 2", 1),
+                            DiscreteDescriptor("b", "x < 3", (), False),
+                            DiscreteDescriptor("c", "x > 15", (), True),
+                            DiscreteDescriptor("d", "y > x", ("foo", "bar"), False),
+                            DiscreteDescriptor("e", "x ** 2 + y == 5", ("foo", "bar"), True),
+                            StringDescriptor("f", "str(x)"),
+                            DateTimeDescriptor("g", "z")
+                        ],
+                        currentIndex=0)
+                )]
+        }
+
+        w = self.create_widget(OWFeatureConstructor, settings)
+        v = [ContinuousVariable(name) for name in "xyz"]
+        domain = Domain(v, [])
+        self.send_signal(w.Inputs.data, Table.from_numpy(domain, [[0, 1, 2]]))
+        w.report_items = Mock()
+        w.send_report()
+        args = w.report_items.call_args[0][1]
+        self.assertEqual(list(args), list("abcdefg"))
+
 
 class TestFeatureEditor(unittest.TestCase):
     def test_has_functions(self):
@@ -363,3 +563,7 @@ class FeatureConstructorHandlerTests(unittest.TestCase):
                 {}
             )
         )
+
+
+if __name__ == "__main__":
+    unittest.main()
