@@ -561,32 +561,32 @@ class TestOWGroupBy(WidgetTest):
         # fmt: off
         expected_df = pd.DataFrame([
             [.15, .15, .1, .07, .005, .3, .1, .2, .1, 0.1, 0.2, 2, 2, 1,
-             "val1", "val2", 2, 2, 1,
+             "val1", "val1", "val2", 2, 2, 1,
              "sval1", "sval2", 2, 2, 1,
              "0.1 0.2", "val1 val2", "sval1 sval2",
              1, 1],
             [.3, .3, .3, np.nan, np.nan, .3, .3, .3, 0, .3, .3, 1, 2, 0.5,
-             "val2", "val2", 1, 2, 0.5,
+             "val2", "val2", "val2", 1, 2, 0.5,
              "", "sval2", 2, 2, 1,
              "0.3", "val2", "sval2",
              1, 2],
             [.433, .4, .3, 0.153, 0.023, 1.3, .3, .6, .3, .3, .6, 3, 3, 1,
-             "val1", "val1", 3, 3, 1,
+             "val1", "val1", "val1", 3, 3, 1,
              "sval1", "sval1", 3, 3, 1,
              "0.3 0.4 0.6", "val1 val2 val1", "sval1 sval2 sval1",
              1, 3],
             [1.5, 1.5, 1, 0.707, 0.5, 3, 1, 2, 1, 1, 2, 2, 2, 1,
-             "val2", "val1", 2, 2, 1,
+             "val1", "val2", "val1", 2, 2, 1,
              "sval2", "sval1", 2, 2, 1,
              "1.0 2.0", "val2 val1", "sval2 sval1",
              2, 1],
             [-0.5, -0.5, -4, 4.95, 24.5, -1, -4, 3, 7, 3, -4, 2, 2, 1,
-             "val2", "val1", 2, 2, 1,
+             "val1", "val2", "val1", 2, 2, 1,
              "sval2", "sval1", 2, 2, 1,
              "3.0 -4.0", "val2 val1", "sval2 sval1",
              2, 2],
             [5, 5, 5, 0, 0, 10, 5, 5, 0, 5, 5, 2, 2, 1,
-             "val2", "val1", 2, 2, 1,
+             "val1", "val2", "val1", 2, 2, 1,
              "sval2", "sval1", 2, 2, 1,
              "5.0 5.0", "val2 val1", "sval2 sval1",
              2, 3]
