@@ -5,6 +5,9 @@ import unittest
 from unittest.mock import Mock, patch
 from AnyQt.QtCore import Qt
 
+from orangewidget.tests.utils import excepthook_catch
+from orangewidget.widget import StateInfo
+
 from Orange.widgets.data.owtable import OWDataTable
 from Orange.widgets.tests.base import WidgetTest, WidgetOutputsTestMixin
 from Orange.data import Table, Domain
@@ -17,7 +20,8 @@ class TestOWDataTable(WidgetTest, WidgetOutputsTestMixin, dbt):
     @classmethod
     def setUpClass(cls):
         super().setUpClass()
-        WidgetOutputsTestMixin.init(cls)
+        WidgetOutputsTestMixin.init(cls,
+                                    output_all_on_no_selection=True)
 
         cls.signal_name = "数据(Data)"
         cls.signal_data = cls.data  # pylint: disable=no-member
@@ -55,8 +59,8 @@ class TestOWDataTable(WidgetTest, WidgetOutputsTestMixin, dbt):
         self.assertListEqual([], self.widget.selected_rows)
 
     def _select_data(self):
-        self.widget.selected_cols = list(range(len(self.data.domain)))
-        self.widget.selected_rows = list(range(0, len(self.data.domain), 10))
+        self.widget.selected_cols = list(range(len(self.data.domain.variables)))
+        self.widget.selected_rows = list(range(0, len(self.data), 10))
         self.widget.set_selection()
         return self.widget.selected_rows
 
@@ -75,7 +79,7 @@ class TestOWDataTable(WidgetTest, WidgetOutputsTestMixin, dbt):
             self.widget.set_corner_text.call_args[0][1], "\na\nb\nc")
 
     def test_unconditional_commit_on_new_signal(self):
-        with patch.object(self.widget, 'unconditional_commit') as commit:
+        with patch.object(self.widget.commit, 'now') as commit:
             self.widget.auto_commit = False
             commit.reset_mock()
             self.send_signal(self.widget.Inputs.data, self.data)
@@ -84,7 +88,7 @@ class TestOWDataTable(WidgetTest, WidgetOutputsTestMixin, dbt):
     def test_pending_selection(self):
         widget = self.create_widget(OWDataTable, stored_settings=dict(
             selected_rows=[5, 6, 7, 8, 9],
-            selected_cols=list(range(len(self.data.domain)))))
+            selected_cols=list(range(len(self.data.domain.variables)))))
         self.send_signal(widget.Inputs.data, None, 1)
         self.send_signal(widget.Inputs.data, self.data, 1)
         output = self.get_output(widget.Outputs.selected_data)
@@ -93,7 +97,7 @@ class TestOWDataTable(WidgetTest, WidgetOutputsTestMixin, dbt):
     def test_sorting(self):
         self.send_signal(self.widget.Inputs.data, self.data)
         self.widget.selected_rows = [0, 1, 2, 3, 4]
-        self.widget.selected_cols = list(range(len(self.data.domain)))
+        self.widget.selected_cols = list(range(len(self.data.domain.variables)))
         self.widget.set_selection()
 
         output = self.get_output(self.widget.Outputs.selected_data)
@@ -105,7 +109,7 @@ class TestOWDataTable(WidgetTest, WidgetOutputsTestMixin, dbt):
         output = self.get_output(self.widget.Outputs.selected_data)
         output, _ = output.get_column_view(0)
         output_sorted = output.tolist()
-        
+
         # the two outputs should not be the same.
         self.assertTrue(output_original != output_sorted)
 
@@ -118,9 +122,9 @@ class TestOWDataTable(WidgetTest, WidgetOutputsTestMixin, dbt):
         info = self.widget.info
         no_input, no_output = "No data on input", "No data on output"
 
-        self.assertEqual(info._StateInfo__input_summary.brief, "-")
+        self.assertIsInstance(info._StateInfo__input_summary, StateInfo.Empty)
         self.assertEqual(info._StateInfo__input_summary.details, no_input)
-        self.assertEqual(info._StateInfo__output_summary.brief, "-")
+        self.assertIsInstance(info._StateInfo__output_summary, StateInfo.Empty)
         self.assertEqual(info._StateInfo__output_summary.details, no_output)
 
         data = Table("zoo")
@@ -128,52 +132,70 @@ class TestOWDataTable(WidgetTest, WidgetOutputsTestMixin, dbt):
         summary, details = f"{len(data)}", format_summary_details(data)
         self.assertEqual(info._StateInfo__input_summary.brief, summary)
         self.assertEqual(info._StateInfo__input_summary.details, details)
-        self.assertEqual(info._StateInfo__output_summary.brief, "-")
-        self.assertEqual(info._StateInfo__output_summary.details, no_output)
-        self.widget.tabs.currentWidget().selectAll()
-        output = self.get_output(self.widget.Outputs.selected_data)
-        summary, details = f"{len(output)}", format_summary_details(output)
-        self.assertEqual(info._StateInfo__output_summary.brief, summary)
-        self.assertEqual(info._StateInfo__output_summary.details, details)
 
         data = Table("iris")
         self.send_signal(self.widget.Inputs.data, data, 2)
         summary, details = f"{len(data)}", format_summary_details(data)
         self.assertEqual(info._StateInfo__input_summary.brief, summary)
         self.assertEqual(info._StateInfo__input_summary.details, details)
-        self.assertEqual(info._StateInfo__output_summary.brief, "-")
-        self.assertEqual(info._StateInfo__output_summary.details, no_output)
-        self._select_data()
-        output = self.get_output(self.widget.Outputs.selected_data)
-        summary, details = f"{len(output)}", format_summary_details(output)
-        self.assertEqual(info._StateInfo__output_summary.brief, summary)
-        self.assertEqual(info._StateInfo__output_summary.details, details)
-        self.widget.tabs.setCurrentWidget(self.widget._inputs[1].view)
-        output = self.get_output(self.widget.Outputs.selected_data)
-        summary, details = f"{len(output)}", format_summary_details(output)
-        self.assertEqual(info._StateInfo__output_summary.brief, summary)
-        self.assertEqual(info._StateInfo__output_summary.details, details)
 
         self.send_signal(self.widget.Inputs.data, None, 1)
         summary, details = f"{len(data)}", format_summary_details(data)
         self.assertEqual(info._StateInfo__input_summary.brief, summary)
         self.assertEqual(info._StateInfo__input_summary.details, details)
-        output = self.get_output(self.widget.Outputs.selected_data)
-        summary, details = f"{len(output)}", format_summary_details(output)
-        self.assertEqual(info._StateInfo__output_summary.brief, summary)
-        self.assertEqual(info._StateInfo__output_summary.details, details)
 
         self.send_signal(self.widget.Inputs.data, None, 2)
-        self.assertEqual(info._StateInfo__input_summary.brief, "-")
+        self.assertIsInstance(info._StateInfo__input_summary, StateInfo.Empty)
         self.assertEqual(info._StateInfo__input_summary.details, no_input)
-        self.assertEqual(info._StateInfo__output_summary.brief, "-")
-        self.assertEqual(info._StateInfo__output_summary.details, no_output)
 
     def test_info(self):
         info_text = self.widget.info_text
-        no_input = "No data."
+        no_input = "无数据."
         self.assertEqual(info_text.text(), no_input)
 
+    def test_show_distributions(self):
+        w = self.widget
+        data = Table("heart_disease")[::3].copy()
+        self.send_signal(w.Inputs.data, data, 0)
+        # run through the delegate paint routines
+        with excepthook_catch():
+            w.grab()
+        w.controls.show_distributions.toggle()
+        with excepthook_catch():
+            w.grab()
+        w.controls.color_by_class.toggle()
+        with excepthook_catch():
+            w.grab()
+        w.controls.show_distributions.toggle()
+
+    def test_whole_rows(self):
+        w = self.widget
+        self.send_signal(w.Inputs.data, self.data, 0)
+        self.assertTrue(w.select_rows)  # default value
+        with excepthook_catch():
+            w.controls.select_rows.toggle()
+        self.assertFalse(w.select_rows)
+        w.selected_cols = [0, 1]
+        w.selected_rows = [0, 1, 2, 3]
+        w.set_selection()
+        out = self.get_output(w.Outputs.selected_data)
+        self.assertEqual(out.domain,
+                         Domain([self.data.domain.attributes[0]], self.data.domain.class_var))
+        with excepthook_catch():
+            w.controls.select_rows.toggle()
+        out = self.get_output(w.Outputs.selected_data)
+        self.assertTrue(w.select_rows)
+        self.assertEqual(out.domain,
+                         self.data.domain)
+
+    def test_show_attribute_labels(self):
+        w = self.widget
+        self.send_signal(w.Inputs.data, self.data, 0)
+        self.assertTrue(w.show_attribute_labels)  # default value
+        with excepthook_catch():
+            w.controls.show_attribute_labels.toggle()
+        self.assertFalse(w.show_attribute_labels)
+
 
 if __name__ == "__main__":
     unittest.main()
