@@ -14,6 +14,7 @@ from Orange.widgets.utils.widgetpreview import WidgetPreview
 
 class OWTreeLearner(OWBaseLearner):
     """Tree algorithm with forward pruning."""
+
     name = "树(Tree)"
     description = "一种前向剪枝的树算法"
     icon = "icons/Tree.svg"
@@ -24,8 +25,8 @@ class OWTreeLearner(OWBaseLearner):
         "Orange.widgets.regression.owregressiontree.OWTreeLearner",
     ]
     priority = 30
-    keywords = ["Classification Tree", 'jueceshu', 'fenleishu', 'shu', 'huiguishu']
-    category = 'model'
+    keywords = ["Classification Tree", "jueceshu", "fenleishu", "shu", "huiguishu"]
+    category = "模型(Model)"
 
     LEARNER = TreeLearner
 
@@ -42,34 +43,56 @@ class OWTreeLearner(OWBaseLearner):
     sufficient_majority = Setting(95)
 
     spin_boxes = (
-        ("叶中的最小实例数: ",
-         "limit_min_leaf", "min_leaf", 1, 1000),
-        ("不要拆分小于以下值的子集: ",
-         "limit_min_internal", "min_internal", 1, 1000),
-        ("将树最大深度限制为: ",
-         "limit_depth", "max_depth", 1, 1000))
+        ("叶中的最小实例数: ", "limit_min_leaf", "min_leaf", 1, 1000),
+        ("不要拆分小于以下值的子集: ", "limit_min_internal", "min_internal", 1, 1000),
+        ("将树最大深度限制为: ", "limit_depth", "max_depth", 1, 1000),
+    )
 
     classification_spin_boxes = (
-        ("当多数达到 [%] 时停止: ",
-         "limit_majority", "sufficient_majority", 51, 100),)
+        ("当多数达到 [%] 时停止: ", "limit_majority", "sufficient_majority", 51, 100),
+    )
 
     def add_main_layout(self):
-        box = gui.widgetBox(self.controlArea, '参数')
+        box = gui.widgetBox(self.controlArea, "参数")
         # the checkbox is put into vBox for alignemnt with other checkboxes
-        gui.checkBox(gui.vBox(box), self, "binary_trees", "归纳二叉树",
-                     callback=self.settings_changed)
+        gui.checkBox(
+            box,
+            self,
+            "binary_trees",
+            "归纳二叉树",
+            callback=self.settings_changed,
+            attribute=Qt.WA_LayoutUsesWidgetRect,
+        )
         for label, check, setting, fromv, tov in self.spin_boxes:
-            gui.spin(box, self, setting, fromv, tov, label=label,
-                     checked=check, alignment=Qt.AlignRight,
-                     callback=self.settings_changed,
-                     checkCallback=self.settings_changed, controlWidth=80)
+            gui.spin(
+                box,
+                self,
+                setting,
+                fromv,
+                tov,
+                label=label,
+                checked=check,
+                alignment=Qt.AlignRight,
+                callback=self.settings_changed,
+                checkCallback=self.settings_changed,
+                controlWidth=80,
+            )
 
     def add_classification_layout(self, box):
         for label, check, setting, minv, maxv in self.classification_spin_boxes:
-            gui.spin(box, self, setting, minv, maxv,
-                     label=label, checked=check, alignment=Qt.AlignRight,
-                     callback=self.settings_changed, controlWidth=80,
-                     checkCallback=self.settings_changed)
+            gui.spin(
+                box,
+                self,
+                setting,
+                minv,
+                maxv,
+                label=label,
+                checked=check,
+                alignment=Qt.AlignRight,
+                callback=self.settings_changed,
+                controlWidth=80,
+                checkCallback=self.settings_changed,
+            )
 
     def learner_kwargs(self):
         # Pylint doesn't get our Settings
@@ -81,7 +104,9 @@ class OWTreeLearner(OWBaseLearner):
             binarize=self.binary_trees,
             preprocessors=self.preprocessors,
             sufficient_majority=(1, self.sufficient_majority / 100)[
-                self.limit_majority])
+                self.limit_majority
+            ],
+        )
 
     def create_learner(self):
         # pylint: disable=not-callable
@@ -89,18 +114,36 @@ class OWTreeLearner(OWBaseLearner):
 
     def get_learner_parameters(self):
         from Orange.widgets.report import plural_w
+
         items = OrderedDict()
-        items["Pruning"] = ", ".join(s for s, c in (
-            (plural_w("at least {number} instance{s} in leaves",
-                      self.min_leaf), self.limit_min_leaf),
-            (plural_w("at least {number} instance{s} in internal nodes",
-                      self.min_internal), self.limit_min_internal),
-            ("maximum depth {}".format(self.max_depth), self.limit_depth)
-        ) if c) or "None"
+        items["Pruning"] = (
+            ", ".join(
+                s
+                for s, c in (
+                    (
+                        plural_w(
+                            "at least {number} instance{s} in leaves", self.min_leaf
+                        ),
+                        self.limit_min_leaf,
+                    ),
+                    (
+                        plural_w(
+                            "at least {number} instance{s} in internal nodes",
+                            self.min_internal,
+                        ),
+                        self.limit_min_internal,
+                    ),
+                    ("maximum depth {}".format(self.max_depth), self.limit_depth),
+                )
+                if c
+            )
+            or "None"
+        )
         if self.limit_majority:
-            items["Splitting"] = "Stop splitting when majority reaches %d%% " \
-                                 "(classification only)" % \
-                                 self.sufficient_majority
+            items["Splitting"] = (
+                "Stop splitting when majority reaches %d%% "
+                "(classification only)" % self.sufficient_majority
+            )
         items["Binary trees"] = ("No", "Yes")[self.binary_trees]
         return items
 
