@@ -1,24 +1,24 @@
+from unicodedata import category
 import warnings
 from collections import namedtuple, OrderedDict
 
 import numpy as np
 
-from AnyQt.QtWidgets import QListView, QApplication
-from AnyQt.QtGui import QBrush, QColor, QPainter
-from AnyQt.QtCore import QEvent
-from orangewidget.utils.listview import ListViewSearch
+from AnyQt.QtWidgets import QListView, QApplication, QSizePolicy
+from AnyQt.QtGui import QBrush, QColor, QPainter, QPalette
+from AnyQt.QtCore import QEvent, Qt
 
 import pyqtgraph as pg
+
+from orangewidget.utils.listview import ListViewSearch
+
 from Orange.data import Table, Domain, ContinuousVariable, StringVariable
 from Orange.statistics import contingency
-
 from Orange.widgets import widget, gui, settings
 from Orange.widgets.utils import itemmodels, colorpalettes
 from Orange.widgets.utils.itemmodels import select_rows
 from Orange.widgets.utils.widgetpreview import WidgetPreview
-from Orange.widgets.utils.state_summary import format_summary_details
-
-from Orange.widgets.visualize.owscatterplotgraph import ScatterPlotItem
+from Orange.widgets.visualize.utils.plotutils import PlotWidget
 from Orange.widgets.widget import Input, Output
 from Orange.widgets.settings import Setting
 
@@ -35,7 +35,7 @@ class OWCorrespondenceAnalysis(widget.OWWidget):
     description = "分类多元数据的对应分析。"
     icon = "icons/CorrespondenceAnalysis.svg"
     keywords = ['duiyingfenxi']
-    category = 'unsupervised'
+    category = '非监督(Unsupervised)'
 
     class Inputs:
         data = Input("数据(Data)", Table, replaces=['Data'])
@@ -63,9 +63,6 @@ class OWCorrespondenceAnalysis(widget.OWWidget):
         self.component_x = 0
         self.component_y = 1
 
-        self._set_input_summary(None)
-        self._set_output_summary(None)
-
         box = gui.vBox(self.controlArea, "变量")
         self.varlist = itemmodels.VariableListModel()
         self.varview = view = ListViewSearch(
@@ -77,26 +74,28 @@ class OWCorrespondenceAnalysis(widget.OWWidget):
 
         box.layout().addWidget(view)
 
-        axes_box = gui.vBox(self.controlArea, "轴线")
-        box = gui.vBox(axes_box, "X 轴", margin=0)
-        box.setFlat(True)
+        axes_box = gui.vBox(self.controlArea, "轴")
         self.axis_x_cb = gui.comboBox(
-            box, self, "component_x", callback=self._component_changed)
+            axes_box, self, "component_x", label="X:",
+            callback=self._component_changed, orientation=Qt.Horizontal,
+            sizePolicy=(QSizePolicy.MinimumExpanding,
+                        QSizePolicy.Preferred)
+        )
 
-        box = gui.vBox(axes_box, "Y 轴", margin=0)
-        box.setFlat(True)
         self.axis_y_cb = gui.comboBox(
-            box, self, "component_y", callback=self._component_changed)
+            axes_box, self, "component_y", label="Y:",
+            callback=self._component_changed, orientation=Qt.Horizontal,
+            sizePolicy=(QSizePolicy.MinimumExpanding,
+                        QSizePolicy.Preferred)
+        )
 
         self.infotext = gui.widgetLabel(
             gui.vBox(self.controlArea, "对惯性的贡献(Contribution to Inertia)"), "\n"
         )
 
-        gui.auto_send(self.controlArea, self, "auto_commit")
+        gui.auto_send(self.buttonsArea, self, "auto_commit")
 
-        gui.rubber(self.controlArea)
-
-        self.plot = pg.PlotWidget(background="w")
+        self.plot = PlotWidget()
         self.plot.setMenuEnabled(False)
         self.mainArea.layout().addWidget(self.plot)
 
@@ -111,7 +110,6 @@ class OWCorrespondenceAnalysis(widget.OWWidget):
             data = None
 
         self.data = data
-        self._set_input_summary(self.data)
         if data is not None:
             self.varlist[:] = [var for var in data.domain.variables
                                if var.is_discrete]
@@ -134,7 +132,9 @@ class OWCorrespondenceAnalysis(widget.OWWidget):
                 self.openContext(data)
                 self._restore_selection()
         self._update_CA()
+        self.commit.now()
 
+    @gui.deferred
     def commit(self):
         output_table = None
         if self.ca is not None:
@@ -151,7 +151,6 @@ class OWCorrespondenceAnalysis(widget.OWWidget):
                               StringVariable("Value")]),
                 rf, metas=vars_data
             )
-        self._set_output_summary(output_table)
         self.Outputs.coordinates.send(output_table)
 
     def clear(self):
@@ -160,16 +159,6 @@ class OWCorrespondenceAnalysis(widget.OWWidget):
         self.plot.clear()
         self.varlist[:] = []
 
-    def _set_input_summary(self, data):
-        summary = len(data) if data else self.info.NoInput
-        details = format_summary_details(data) if data else ""
-        self.info.set_input_summary(summary, details)
-
-    def _set_output_summary(self, output):
-        summary = len(output) if output else self.info.NoOutput
-        details = format_summary_details(output) if output else ""
-        self.info.set_output_summary(summary, details)
-
     def selected_vars(self):
         rows = sorted(
             ind.row() for ind in self.varview.selectionModel().selectedRows())
@@ -209,6 +198,7 @@ class OWCorrespondenceAnalysis(widget.OWWidget):
             self.ca = None
             self.plot.clear()
             self._update_CA()
+            self.commit.deferred()
             return
         return super().customEvent(event)
 
@@ -223,7 +213,6 @@ class OWCorrespondenceAnalysis(widget.OWWidget):
 
         self._setup_plot()
         self._update_info()
-        self.commit()
 
     def update_XY(self):
         self.axis_x_cb.clear()
@@ -290,6 +279,7 @@ class OWCorrespondenceAnalysis(widget.OWWidget):
         margin = margin * 0.05 if margin > 1e-10 else 1
         self.plot.setYRange(minmax[2] - margin, minmax[3] + margin)
 
+        foreground = self.palette().color(QPalette.Text)
         for i, (v, points) in enumerate(zip(variables, points)):
             color_outline = colors[i]
             color_outline.setAlpha(200)
@@ -303,7 +293,7 @@ class OWCorrespondenceAnalysis(widget.OWWidget):
             self.plot.addItem(item)
 
             for name, point in zip(v.values, points):
-                item = pg.TextItem(name, anchor=(0.5, 0))
+                item = pg.TextItem(name, anchor=(0.5, 0), color=foreground)
                 self.plot.addItem(item)
                 item.setPos(point[0], point[1])
 
