@@ -21,12 +21,14 @@ from Orange.data.domain import filter_visible
 from Orange.widgets import widget, gui, settings
 from Orange.widgets.utils import itemmodels, colorpalettes
 from Orange.widgets.utils.annotated_data import (create_annotated_table,
-                                                 ANNOTATED_DATA_SIGNAL_Chinese_NAME)
+                                                 ANNOTATED_DATA_SIGNAL_NAME)
+from Orange.widgets.utils.graphicsscene import graphicsscene_help_event
 from Orange.widgets.utils.graphicstextlist import TextListWidget
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.visualize.utils.plotutils import HelpEventDelegate
 from Orange.widgets.widget import Input, Output
 from Orange.widgets.utils.dendrogram import DendrogramWidget
-from Orange.widgets.utils.state_summary import format_summary_details
+from Orange.widgets.visualize.utils.plotutils import GraphicsView
 from Orange.widgets.visualize.utils.heatmap import (
     GradientColorMap, GradientLegendWidget,
 )
@@ -246,20 +248,32 @@ class DistanceMapItem(pg.ImageItem):
             self.setToolTip("")
 
 
+class GraphicsView(pg.GraphicsView):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        scene = self.scene()
+        delegate = HelpEventDelegate(self.__helpEvent, parent=self)
+        scene.installEventFilter(delegate)
+
+    def __helpEvent(self, event):
+        graphicsscene_help_event(self.scene(), event)
+        return event.isAccepted()
+
+
 class OWDistanceMap(widget.OWWidget):
     name = "距离图(Distance Map)"
     description = "可视化距离矩阵"
     icon = "icons/DistanceMap.svg"
     priority = 1200
     keywords = ['juliyingshe', 'yingshe', 'julitu']
-    category = 'unsupervised'
+    category = '非监督(Unsupervised)'
 
     class Inputs:
         distances = Input("距离(Distances)", Orange.misc.DistMatrix, replaces=['Distances'])
 
     class Outputs:
         selected_data = Output("选定的数据(Selected Data)", Orange.data.Table, default=True, replaces=['Selected Data'])
-        annotated_data = Output(ANNOTATED_DATA_SIGNAL_Chinese_NAME, Orange.data.Table, replaces=['Data'])
+        annotated_data = Output("数据(Data)", Orange.data.Table, replaces=['Data'])
         features = Output("特征(Features)", widget.AttributeList, dynamic=False, replaces=['Features'])
 
     settingsHandler = settings.PerfectDomainContextHandler()
@@ -297,9 +311,6 @@ class OWDistanceMap(widget.OWWidget):
         self._sort_indices = None
         self._selection = None
 
-        self._set_input_summary(None)
-        self._set_output_summary(None)
-
         self.sorting_cb = gui.comboBox(
             self.controlArea, self, "sorting", box="元素排序",
             items=["无", "聚类(Clustering)", "有序叶聚类"],
@@ -331,11 +342,11 @@ class OWDistanceMap(widget.OWWidget):
         )
         self.annot_combo.setModel(itemmodels.VariableListModel())
         self.annot_combo.model()[:] = ["无", "枚举"]
-        self.controlArea.layout().addStretch()
+        gui.rubber(self.controlArea)
 
-        gui.auto_send(self.controlArea, self, "autocommit")
+        gui.auto_send(self.buttonsArea, self, "autocommit")
 
-        self.view = pg.GraphicsView(background="w")
+        self.view = GraphicsView(background=None)
         self.mainArea.layout().addWidget(self.view)
 
         self.grid_widget = pg.GraphicsWidget()
@@ -407,7 +418,6 @@ class OWDistanceMap(widget.OWWidget):
         self.closeContext()
         self.clear()
         self.error()
-        self._set_input_summary(matrix)
         if matrix is not None:
             N, _ = matrix.shape
             if N < 2:
@@ -487,16 +497,7 @@ class OWDistanceMap(widget.OWWidget):
             if self.pending_selection is not None:
                 self.matrix_item.set_selections(self.pending_selection)
                 self.pending_selection = None
-        self.unconditional_commit()
-
-    def _set_input_summary(self, matrix):
-        summary = len(matrix) if matrix is not None else self.info.NoInput
-        self.info.set_input_summary(summary)
-
-    def _set_output_summary(self, output):
-        summary = len(output) if output else self.info.NoOutput
-        details = format_summary_details(output) if output else ""
-        self.info.set_output_summary(summary, details)
+        self.commit.now()
 
     def _clear_plot(self):
         def remove(item):
@@ -646,8 +647,9 @@ class OWDistanceMap(widget.OWWidget):
             sortind = self._sort_indices
             indices = [sortind[i] for i in indices]
         self._selection = list(sorted(set(indices)))
-        self.commit()
+        self.commit.deferred()
 
+    @gui.deferred
     def commit(self):
         datasubset = None
         featuresubset = None
@@ -668,8 +670,6 @@ class OWDistanceMap(widget.OWWidget):
             subset = [self.items[i] for i in self._selection]
             featuresubset = widget.AttributeList(subset)
 
-        self._set_output_summary(datasubset)
-
         self.Outputs.selected_data.send(datasubset)
         self.Outputs.annotated_data.send(create_annotated_table(self.items, self._selection))
         self.Outputs.features.send(featuresubset)
