@@ -9,28 +9,17 @@ import numpy as np
 import sklearn.metrics
 
 from AnyQt.QtWidgets import (
-    QGraphicsWidget,
-    QGraphicsGridLayout,
-    QGraphicsRectItem,
-    QStyleOptionGraphicsItem,
-    QSizePolicy,
-    QWidget,
-    QVBoxLayout,
-    QGraphicsSimpleTextItem,
-    QWIDGETSIZE_MAX,
-    QGraphicsSceneHelpEvent,
-    QToolTip,
-    QApplication,
+    QGraphicsWidget, QGraphicsGridLayout,
+    QGraphicsRectItem, QStyleOptionGraphicsItem, QSizePolicy, QWidget,
+    QVBoxLayout, QGraphicsSimpleTextItem, QWIDGETSIZE_MAX,
+    QGraphicsSceneHelpEvent, QToolTip, QApplication,
 )
 from AnyQt.QtGui import (
-    QColor,
-    QPen,
-    QBrush,
-    QPainter,
-    QFontMetrics,
-    QPalette,
+    QColor, QPen, QBrush, QPainter, QFontMetrics, QPalette,
+)
+from AnyQt.QtCore import (
+    Qt, QEvent, QRectF, QSizeF, QSize, QPointF, QPoint, QRect
 )
-from AnyQt.QtCore import Qt, QEvent, QRectF, QSizeF, QSize, QPointF, QPoint, QRect
 from AnyQt.QtCore import pyqtSignal as Signal
 
 import Orange.data
@@ -44,10 +33,8 @@ from Orange.widgets import widget, gui, settings
 from Orange.widgets.utils.graphicsscene import GraphicsScene
 from Orange.widgets.utils.stickygraphicsview import StickyGraphicsView
 from Orange.widgets.utils import itemmodels, apply_all
-from Orange.widgets.utils.annotated_data import (
-    create_annotated_table,
-    ANNOTATED_DATA_SIGNAL_NAME,
-)
+from Orange.widgets.utils.annotated_data import (create_annotated_table,
+                                                 ANNOTATED_DATA_SIGNAL_NAME)
 from Orange.widgets.utils.graphicstextlist import TextListWidget
 from Orange.widgets.utils.graphicslayoutitem import SimpleLayoutItem
 from Orange.widgets.utils.sql import check_sql_input
@@ -98,14 +85,15 @@ class OWSilhouettePlot(widget.OWWidget):
         "Orange.widgets.unsupervised.owsilhouetteplot.OWSilhouettePlot",
     ]
 
-    settingsHandler = settings.PerfectDomainContextHandler()
+    settingsHandler = settings.DomainContextHandler()
+    settings_version = 2
 
     #: Distance metric index
     distance_idx = settings.Setting(0)
-    #: Group/cluster variable index
-    cluster_var_idx = settings.ContextSetting(0)
-    #: Annotation variable index
-    annotation_var_idx = settings.ContextSetting(0)
+    #: Group/cluster variable
+    cluster_var = settings.ContextSetting(None)
+    #: Annotation variable
+    annotation_var = settings.ContextSetting(None)
     #: Group the (displayed) silhouettes by cluster
     group_by_cluster = settings.Setting(True)
     #: A fixed size for an instance bar
@@ -170,47 +158,30 @@ class OWSilhouettePlot(widget.OWWidget):
         )
         controllayout.addWidget(distbox)
 
-        box = gui.vBox(self.controlArea, "簇标签")
+        box = gui.vBox(self.controlArea, "分组")
+        self.cluster_var_model = itemmodels.VariableListModel(
+            parent=self, placeholder="(无)")
         self.cluster_var_cb = gui.comboBox(
-            box,
-            self,
-            "cluster_var_idx",
-            contentsLength=14,
-            searchable=True,
-            callback=self._invalidate_scores,
+            box, self, "cluster_var", contentsLength=14,
+            searchable=True, callback=self._invalidate_scores,
+            model=self.cluster_var_model
         )
         gui.checkBox(
-            box,
-            self,
-            "group_by_cluster",
-            "按簇分组(Group by cluster)",
-            callback=self._replot,
-        )
-        self.cluster_var_model = itemmodels.VariableListModel(parent=self)
-        self.cluster_var_cb.setModel(self.cluster_var_model)
+            box, self, "group_by_cluster", "分组显示",
+            callback=self._replot)
 
         box = gui.vBox(self.controlArea, "条")
         gui.widgetLabel(box, "条宽度:")
         gui.hSlider(
-            box,
-            self,
-            "bar_size",
-            minValue=1,
-            maxValue=10,
-            step=1,
-            callback=self._update_bar_size,
-        )
+            box, self, "bar_size", minValue=1, maxValue=10, step=1,
+            callback=self._update_bar_size)
         gui.widgetLabel(box, "标注:")
-        self.annotation_cb = gui.comboBox(
-            box,
-            self,
-            "annotation_var_idx",
-            contentsLength=14,
-            callback=self._update_annotations,
-        )
         self.annotation_var_model = itemmodels.VariableListModel(parent=self)
-        self.annotation_var_model[:] = ["无"]
-        self.annotation_cb.setModel(self.annotation_var_model)
+        self.annotation_var_model[:] = [None]
+        self.annotation_cb = gui.comboBox(
+            box, self, "annotation_var", contentsLength=14,
+            callback=self._update_annotations,
+            model=self.annotation_var_model)
         ibox = gui.indentedBox(box, 5)
         self.ann_hidden_warning = warning = gui.widgetLabel(ibox, "(增加要显示的宽度)")
         ibox.setFixedWidth(ibox.sizeHint().width())
@@ -301,17 +272,14 @@ class OWSilhouettePlot(widget.OWWidget):
             raise NoGroupVariable()
         self.cluster_var_model[:] = groupvars
         if domain.class_var in groupvars:
-            self.cluster_var_idx = groupvars.index(domain.class_var)
+            self.cluster_var = domain.class_var
         else:
-            self.cluster_var_idx = 0
-        annotvars = [
-            var
-            for var in domain.variables + domain.metas
-            if var.is_string or var.is_discrete
-        ]
-        self.annotation_var_model[:] = ["无"] + annotvars
-        self.annotation_var_idx = 1 if annotvars else 0
-        self.openContext(Orange.data.Domain(groupvars))
+            self.cluster_var = groupvars[0]
+        annotvars = [var for var in domain.variables + domain.metas
+                     if var.is_string or var.is_discrete]
+        self.annotation_var_model[:] = [None] + annotvars
+        self.annotation_var = annotvars[0] if annotvars else None
+        self.openContext(domain)
 
     def _is_empty(self) -> bool:
         # Is empty (does not have any input).
@@ -392,8 +360,7 @@ class OWSilhouettePlot(widget.OWWidget):
         if self._matrix is None:
             return
 
-        labelvar = self.cluster_var_model[self.cluster_var_idx]
-        labels, _ = self.data.get_column_view(labelvar)
+        labels, _ = self.data.get_column_view(self.cluster_var)
         labels = np.asarray(labels, dtype=float)
         cluster_mask = np.isnan(labels)
         dist_mask = np.isnan(self._matrix).all(axis=0)
@@ -444,20 +411,20 @@ class OWSilhouettePlot(widget.OWWidget):
         visible = self.bar_size >= 5
         self._silplot.setBarHeight(self.bar_size)
         self._silplot.setRowNamesVisible(visible)
-        self.ann_hidden_warning.setVisible(not visible and self.annotation_var_idx > 0)
+        self.ann_hidden_warning.setVisible(
+            not visible and self.annotation_var is not None)
 
     def _replot(self):
         # Clear and replot/initialize the scene
         self._clear_scene()
         if self._silhouette is not None and self._labels is not None:
-            var = self.cluster_var_model[self.cluster_var_idx]
             self._silplot = silplot = SilhouettePlot()
             self._set_bar_height()
 
             if self.group_by_cluster:
                 silplot.setScores(
-                    self._silhouette, self._labels, var.values, var.colors
-                )
+                    self._silhouette, self._labels,
+                    self.cluster_var.values, self.cluster_var.colors)
             else:
                 silplot.setScores(
                     self._silhouette,
@@ -478,11 +445,9 @@ class OWSilhouettePlot(widget.OWWidget):
             self._set_bar_height()
 
     def _update_annotations(self):
-        if 0 < self.annotation_var_idx < len(self.annotation_var_model):
-            annot_var = self.annotation_var_model[self.annotation_var_idx]
-        else:
-            annot_var = None
-        self.ann_hidden_warning.setVisible(self.bar_size < 5 and annot_var is not None)
+        annot_var = self.annotation_var
+        self.ann_hidden_warning.setVisible(
+            self.bar_size < 5 and annot_var is not None)
 
         if self._silplot is not None:
             if annot_var is not None:
@@ -545,21 +510,17 @@ class OWSilhouettePlot(widget.OWWidget):
             else:
                 scores = self._silhouette
 
-            var = self.cluster_var_model[self.cluster_var_idx]
-
             domain = self.data.domain
-            proposed = "Silhouette ({})".format(escape(var.name))
-            names = [
-                var.name
-                for var in itertools.chain(
-                    domain.attributes, domain.class_vars, domain.metas
-                )
-            ]
+            proposed = "Silhouette ({})".format(escape(self.cluster_var.name))
+            names = [var.name for var in itertools.chain(domain.attributes,
+                                                         domain.class_vars,
+                                                         domain.metas)]
             unique = get_unique_names(names, proposed)
             silhouette_var = Orange.data.ContinuousVariable(unique)
             domain = Orange.data.Domain(
-                domain.attributes, domain.class_vars, domain.metas + (silhouette_var,)
-            )
+                domain.attributes,
+                domain.class_vars,
+                domain.metas + (silhouette_var, ))
 
             if np.count_nonzero(selectedmask):
                 selected = self.data.from_table(
@@ -582,32 +543,45 @@ class OWSilhouettePlot(widget.OWWidget):
             return
 
         self.report_plot()
-        caption = "Silhouette plot ({} distance), clustered by '{}'".format(
-            self.Distances[self.distance_idx][0],
-            self.cluster_var_model[self.cluster_var_idx],
-        )
-        if self.annotation_var_idx and self._silplot.rowNamesVisible():
-            caption += ", annotated with '{}'".format(
-                self.annotation_var_model[self.annotation_var_idx]
-            )
+        caption = "Silhouette plot " \
+                  f"({self.Distances[self.distance_idx][0]} distance), " \
+                  f"clustered by '{self.cluster_var.name}'"
+        if self.annotation_var and self._silplot.rowNamesVisible():
+            caption += f", annotated with '{self.annotation_var.name}'"
         self.report_caption(caption)
 
     def onDeleteWidget(self):
         self.clear()
         super().onDeleteWidget()
 
+    @classmethod
+    def migrate_context(cls, context, version):
+        values = context.values
+        if version < 2:
+            # contexts were constructed from Domain containing vars shown in
+            # the list view, context.class_vars and context.metas were always
+            # empty, and context.attributes contained discrete attributes
+            index, _ = values.pop("cluster_var_idx")
+            values["cluster_var"] = (context.attributes[index][0], 101)
+
+            index = values.pop("annotation_var_idx")[0] - 1
+            if index == -1:
+                values["annotation_var"] = None
+            elif index < len(context.attributes):
+                name, _ = context.attributes[index]
+                values["annotation_var"] = (name, 101)
+            # else we cannot migrate
+            # Even this migration can be erroneous if metas contained a mixture
+            # of discrete and string attributes; the latter were not stored in
+            # context, so indices in context could have been wrong
+
 
 class SelectAction(enum.IntEnum):
     NoUpdate, Clear, Select, Deselect, Toogle, Current = 1, 2, 4, 8, 16, 32
 
 
-def show_tool_tip(
-    pos: QPoint,
-    text: str,
-    widget: Optional[QWidget] = None,
-    rect=QRect(),
-    elide=Qt.ElideRight,
-):
+def show_tool_tip(pos: QPoint, text: str, widget: Optional[QWidget] = None,
+                  rect=QRect(), elide=Qt.ElideRight):
     """
     Show a plain text tool tip with limited length, eliding if necessary.
     """
@@ -634,7 +608,6 @@ class StyledGraphicsView(StickyGraphicsView):
     """
     Propagate style and palette changes to the visualized scene.
     """
-
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.ensurePolished()
@@ -647,11 +620,8 @@ class StyledGraphicsView(StickyGraphicsView):
             self.scene().setPalette(self.palette())
 
     def changeEvent(self, event):
-        if (
-            event.type() == QEvent.PaletteChange
-            and self.scene() is not None
-            and self.scene().parent() is self
-        ):
+        if event.type() == QEvent.PaletteChange and \
+                self.scene() is not None and self.scene().parent() is self:
             self.scene().setPalette(self.palette())
         super().changeEvent(event)
 
@@ -676,9 +646,9 @@ class SilhouettePlot(QGraphicsWidget):
         self.__pen = QPen(Qt.NoPen)
         self.__layout = QGraphicsGridLayout()
         self.__hoveredItem = None
-        self.__topScale = None  # type: Optional[AxisItem]
+        self.__topScale = None     # type: Optional[AxisItem]
         self.__bottomScale = None  # type: Optional[AxisItem]
-        self.__layout.setColumnSpacing(0, 1.0)
+        self.__layout.setColumnSpacing(0, 1.)
         self.setLayout(self.__layout)
         self.setFocusPolicy(Qt.StrongFocus)
 
@@ -830,9 +800,10 @@ class SilhouettePlot(QGraphicsWidget):
 
             if group.label:
                 layout.addItem(Line(orientation=Qt.Vertical), i + 1, 1)
-                label = QGraphicsSimpleTextItem(
-                    "{} ({})".format(group.label, len(group.scores)), self
-                )
+                text = group.label
+                if group.scores.size:
+                    text += f" ({np.mean(group.scores):.3f})"
+                label = QGraphicsSimpleTextItem(text, self)
                 label.setBrush(foreground)
                 label.setPen(QPen(Qt.NoPen))
                 label.setRotation(-90)
@@ -845,10 +816,8 @@ class SilhouettePlot(QGraphicsWidget):
                 layout.addItem(item, i + 1, 0, Qt.AlignCenter)
 
             textlist = _SilhouettePlotTextListWidget(
-                self,
-                font=font,
-                elideMode=Qt.ElideRight,
-                alignment=Qt.AlignLeft | Qt.AlignVCenter,
+                self, font=font, elideMode=Qt.ElideRight,
+                alignment=Qt.AlignLeft | Qt.AlignVCenter
             )
             textlist.setMaximumWidth(750)
             textlist.setFlag(TextListWidget.ItemClipsChildrenToShape, False)
@@ -883,13 +852,8 @@ class SilhouettePlot(QGraphicsWidget):
             height = silitem.effectiveSizeHint(Qt.PreferredSize).height()
             textitem.setMaximumHeight(height)
             textitem.setMinimumHeight(height)
-        mwidth = max(
-            (
-                silitem.effectiveSizeHint(Qt.PreferredSize).width()
-                for silitem in self.__plotItems()
-            ),
-            default=300,
-        )
+        mwidth = max((silitem.effectiveSizeHint(Qt.PreferredSize).width()
+                     for silitem in self.__plotItems()), default=300)
         # match the AxisItem's width to the bars
         for axis in self.__axisItems():
             axis.setMaximumWidth(mwidth)
@@ -898,17 +862,15 @@ class SilhouettePlot(QGraphicsWidget):
     def changeEvent(self, event: QEvent) -> None:
         if event.type() == QEvent.PaletteChange:
             brush = self.palette().brush(QPalette.Text)
-            labels = [
-                it
-                for it in self.childItems()
-                if isinstance(it, QGraphicsSimpleTextItem)
-            ]
+            labels = [it for it in self.childItems()
+                      if isinstance(it, QGraphicsSimpleTextItem)]
             apply_all(labels, lambda it: it.setBrush(brush))
         super().changeEvent(event)
 
     def event(self, event: QEvent) -> bool:
         # Reimplemented
-        if event.type() == QEvent.LayoutRequest and self.parentLayoutItem() is None:
+        if event.type() == QEvent.LayoutRequest and \
+                self.parentLayoutItem() is None:
             self.__updateSizeConstraints()
             self.resize(self.effectiveSizeHint(Qt.PreferredSize))
         elif event.type() == QEvent.GraphicsSceneHelp:
@@ -1100,21 +1062,17 @@ class SilhouettePlot(QGraphicsWidget):
             return None
 
     def __itemDataAtPos(self, pos) -> Optional[Tuple[namespace, int, QRectF]]:
-        items = [
-            (sitem, tlist, grp)
-            for sitem, tlist, grp in zip(
-                self.__plotItems(), self.__textItems(), self.__groups
-            )
-            if sitem.geometry().contains(pos)
-            or tlist.isVisible()
-            and tlist.geometry().contains(pos)
-        ]
+        items = [(sitem, tlist, grp) for sitem, tlist, grp
+                 in zip(self.__plotItems(), self.__textItems(), self.__groups)
+                 if sitem.geometry().contains(pos) or tlist.isVisible()
+                 and tlist.geometry().contains(pos)]
         if not items:
             return None
         else:
             sitem, _, grp = items[0]
         indices = grp.indices
-        assert isinstance(indices, np.ndarray) and indices.shape == (sitem.count(),)
+        assert (isinstance(indices, np.ndarray) and
+                indices.shape == (sitem.count(),))
         crect = sitem.contentsRect()
         pos = sitem.mapFromParent(pos)
         if not crect.top() <= pos.y() <= crect.bottom():
