@@ -418,6 +418,10 @@ class OWTreeViewer2D(OWWidget, openclass=True):
 
         self.scene = TreeGraphicsScene(self)
         self.scene_view = TreeGraphicsView(self.scene)
+        self.scene_view.setStyleSheet("""QToolTip { padding: 3px;
+                                                    border: 1px solid #C0C0C0;
+                                      }""")
+
         self.scene_view.setViewportUpdateMode(QGraphicsView.FullViewportUpdate)
         self.mainArea.layout().addWidget(self.scene_view)
         self.toggle_zoom_slider()
